# -*- coding: utf-8 -*-

import os
import json
from datetime import datetime, timedelta

import pandas as pd

from ydl_ai_recommender.src.utils.log import create_logger
from ydl_ai_recommender.src.core.manager import DatabaseDataManager


class ChatDataManager(DatabaseDataManager):
    def __init__(self, client=None) -> None:
        super().__init__(client, create_logger(__name__, 'chat_data_manager.log'))
        self.now = datetime.now()


    def _make_query_sql(self, conditions=None):

        if conditions:
            conditions = ['doctor_id IS NOT NULL'] + conditions
        else:
            conditions = ['doctor_id IS NOT NULL']
        
        condition_sql = ' WHERE ' + ' AND '.join(conditions)

        select_fields = ['dt', 'uid', 'doctor_id', 'user_to_doctor', 'doctor_to_user']
        sql = 'SELECT {} FROM dws.dws_user_chat_assistant_doctor_day'.format(', '.join(select_fields))
        sql += condition_sql
        return sql


    def update_data(self):
        """ 从数据库中拉取最新订单数据并保存 """
        sql = self._make_query_sql()
        _, all_data = self.fetch_data_from_db(sql)

        df = pd.DataFrame(all_data)
        self.save_csv_data(df, 'all_chat_info.csv')
        return df


    def update_test_data(self, conditions):
        """ 从数据库中拉取指定条件订单用于测试 """

        sql = self._make_query_sql(conditions)
        _, all_data = self.fetch_data_from_db(sql)

        df = pd.DataFrame(all_data)
        self.save_csv_data(df, 'test_chat_info.csv')


    def load_raw_data(self):
        return self.load_csv_data('all_chat_info.csv')


    def load_test_data(self):
        return self.load_csv_data('test_chat_info.csv')


    def make_index(self):
        """
        构建索引
            用户-咨询师 索引
        """
        self.logger.info('')
        self.logger.info('开始构建 用户-咨询师 索引')
        
        df = self.load_raw_data()
        self.logger.info('本地用户咨询师对话数据加载完成，共加载 %s 条数据', len(df))

        user_chat = {}
        for index, row in df.iterrows():
            uid, supplier_id = row['uid'], row['doctor_id']
        
            if uid not in user_chat:
                user_chat[uid] = {}
            if supplier_id not in user_chat[uid]:
                user_chat[uid][supplier_id] = []

            user_chat[uid][supplier_id].append([row['dt'], row['user_to_doctor'], row['doctor_to_user']])


        def compute_score(infos):
            w = [0, 0, 0, 0]
            for [dt, _u2d, _d2u] in infos:
                u2d, d2u = int(_u2d), int(_d2u)
                date = datetime.strptime(dt, '%Y-%m-%d')

                if (self.now - date) <= timedelta(days=7):
                    w[0] = max(1., w[0], (u2d + d2u) / 20)
                elif (self.now - date) <= timedelta(days=30):
                    w[1] = max(1., w[1], (u2d + d2u) / 20)
                elif (self.now - date) <= timedelta(days=180):
                    w[2] = max(1., w[2], (u2d + d2u) / 20)
                else:
                    w[3] = max(1., w[3], (u2d + d2u) / 20)

            value = w[0] * 0.5 + w[1] * 0.25 + w[2] * 0.15 + w[3] * 0.1
            return value

        index = {}
        for uid, chats in user_chat.items():
            supplier_values = []
            for supplier_id, infos in chats.items():
                # 日期越近权重越大
                value = compute_score(infos)
                supplier_values.append([supplier_id, value])
            index[uid] = sorted(supplier_values, key=lambda x: x[1], reverse=True)
        
        self.logger.info('用户-咨询师 询单索引构建完成，共构建 %s 条数据', len(index))
        with open(os.path.join(self.local_file_dir, 'user_doctor_chat_index.json'), 'w', encoding='utf-8') as f:
            json.dump(index, f, ensure_ascii=False)


if __name__ == '__main__':
    from ydl_ai_recommender.src.data.mysql_client import MySQLClient
    from ydl_ai_recommender.src.utils import get_conf_path


    client = MySQLClient.create_from_config_file(get_conf_path())
    manager = ChatDataManager(client)
    # manager.update_data()
    manager.make_index()