# -*- coding: utf-8 -*-

import os
import json
from collections import Counter
from datetime import datetime, timedelta

import pandas as pd

from ydl_ai_recommender.src.utils.log import create_logger
from ydl_ai_recommender.src.core.manager import DatabaseDataManager


class OrderDataManager(DatabaseDataManager):
    def __init__(self, client=None) -> None:
        super().__init__(client, create_logger(__name__, 'order_data_manager.log'))
        self.now = datetime.now()


    def _make_query_sql(self, conditions=None):
        condition_sql = ''
        if conditions:
            condition_sql = ' WHERE ' + ' AND '.join(conditions)

        select_fields = ['main_order_id', 'uid', 'supplier_id', 'price', 'standard_order_type']
        select_fields.append('DATE_FORMAT(update_time, "%Y-%m-%d") AS update_time')
        sql = 'SELECT {} FROM ods.ods_ydl_standard_order'.format(', '.join(select_fields))
        sql += condition_sql
        return sql


    def update_order_data(self):
        """ 从数据库中拉取最新订单数据并保存 """
        sql = self._make_query_sql()
        _, all_data = self.fetch_data_from_db(sql)

        df = pd.DataFrame(all_data)
        self.save_xlsx_data(df, 'all_order_info.xlsx')


    def update_test_order_data(self, conditions):
        """ 从数据库中拉取指定条件订单用于测试 """

        sql = self._make_query_sql(conditions)
        _, all_data = self.fetch_data_from_db(sql)

        df = pd.DataFrame(all_data)
        self.save_xlsx_data(df, 'test_order_info.xlsx')


    def load_raw_data(self):
        return self.load_xlsx_data('all_order_info.xlsx')


    def load_test_order_data(self):
        return self.load_xlsx_data('test_order_info.xlsx')


    def make_index(self):
        """
        构建索引
            用户-咨询师 索引
            top100 咨询师列表 用于冷启动
        """
        self.logger.info('')
        self.logger.info('开始构建 用户-咨询师 索引')
        
        df = self.load_raw_data()
        self.logger.info('本地订单加载数据完成，共加载 %s 条数据', len(df))

        user_order = {}
        for index, row in df.iterrows():
            uid, supplier_id = row['uid'], row['supplier_id']
        
            if uid not in user_order:
                user_order[uid] = {}
            if supplier_id not in user_order[uid]:
                user_order[uid][supplier_id] = []
            user_order[uid][supplier_id].append([row['price'], row['update_time']])
        
        
        def compute_score(infos):
            w = [0, 0, 0, 0]
            for [_price, dt] in infos:
                price = float(_price)
                date = datetime.strptime(dt, '%Y-%m-%d')

                if (self.now - date) <= timedelta(days=7):
                    w[0] = max(1., w[0], price / 400)
                elif (self.now - date) <= timedelta(days=30):
                    w[1] = max(1., w[1], price / 400)
                elif (self.now - date) <= timedelta(days=180):
                    w[2] = max(1., w[2], price / 400)
                else:
                    w[3] = max(1., w[3], price / 400)

            value = w[0] * 0.5 + w[1] * 0.25 + w[2] * 0.15 + w[3] * 0.1
            return value

        
        index = {}
        for uid, orders in user_order.items():
            supplier_values = []
            for supplier_id, infos in orders.items():
                # 订单越多排序约靠前，相同数量订单，最新订单约晚越靠前
                value = compute_score(infos)
                supplier_values.append([supplier_id, value])

            #     value = len(infos)
            #     latest_time = max([info[1] for info in infos])
            #     supplier_values.append([supplier_id, value, latest_time])
            
            # index[uid] = sorted(supplier_values, key=lambda x: (x[2], x[1]), reverse=True)

            index[uid] = sorted(supplier_values, key=lambda x: x[1], reverse=True)
        
        self.logger.info('用户-咨询师 索引构建完成，共构建 %s 条数据', len(index))
        with open(os.path.join(self.local_file_dir, 'user_doctor_index.json'), 'w', encoding='utf-8') as f:
            json.dump(index, f, ensure_ascii=False)

        self.logger.info('用户-咨询师 索引数据已保存，共有用户 %s', len(index))
        
        # 订单最多的咨询师
        supplier_cnter = Counter(df['supplier_id'])
        top100_supplier = []
        for key, _ in supplier_cnter.most_common(100):
            top100_supplier.append(str(key))

        self.logger.info('top100 订单量咨询师统计完成')
        with open(os.path.join(self.local_file_dir, 'top100_supplier.txt'), 'w', encoding='utf-8') as f:
            f.write('\n'.join(top100_supplier))
        self.logger.info('top100 订单量咨询师列表已保存')


if __name__ == '__main__':
    manager = OrderDataManager()
    print(manager.make_index())